@@ -33,6 +33,7 @@
 #include <osmocom/vty/command.h>
 #include <osmocom/vty/vty.h>
 #include <osmocom/vty/misc.h>
+#include <osmocom/vty/tdef_vty.h>
 
 #include "../gtp/gtp.h"
 #include "../gtp/pdp.h"
@@ -298,7 +299,7 @@ DEFUN(cfg_ggsn_no_default_apn, cfg_ggsn_no_default_apn_cmd,
 
 DEFUN(cfg_ggsn_shutdown, cfg_ggsn_shutdown_cmd,
 	"shutdown ggsn",
-	"Put the GGSN in administrative shut-down\n" GGSN_STR)
+	"Put the GGSN in administrative shutdown\n" GGSN_STR)
 {
 	struct ggsn_ctx *ggsn = (struct ggsn_ctx *) vty->index;
 
@@ -315,7 +316,7 @@ DEFUN(cfg_ggsn_shutdown, cfg_ggsn_shutdown_cmd,
 
 DEFUN(cfg_ggsn_no_shutdown, cfg_ggsn_no_shutdown_cmd,
 	"no shutdown ggsn",
-	NO_STR GGSN_STR "Remove the GGSN from administrative shut-down\n")
+	NO_STR GGSN_STR "Remove the GGSN from administrative shutdown\n")
 {
 	struct ggsn_ctx *ggsn = (struct ggsn_ctx *) vty->index;
 
@@ -343,7 +344,7 @@ static void show_one_sgsn(struct vty *vty, const struct sgsn_peer *sgsn, const c
 
 DEFUN(cfg_ggsn_show_sgsn, cfg_ggsn_show_sgsn_cmd,
 	"show sgsn",
-	NO_STR GGSN_STR "Remove the GGSN from administrative shut-down\n")
+	NO_STR GGSN_STR "Remove the GGSN from administrative shutdown\n")
 {
 	struct ggsn_ctx *ggsn = (struct ggsn_ctx *) vty->index;
 	struct sgsn_peer *sgsn;
@@ -389,10 +390,9 @@ DEFUN(cfg_ggsn_no_echo_interval, cfg_ggsn_no_echo_interval_cmd,
 	NO_STR "Send an echo request to this static GGSN every interval.\n")
 {
 	struct ggsn_ctx *ggsn = (struct ggsn_ctx *) vty->index;
-	int prev_interval = ggsn->cfg.echo_interval;
 	struct sgsn_peer *sgsn;
 
-	if (prev_interval == ggsn->cfg.echo_interval)
+	if (ggsn->cfg.echo_interval == 0)
 		return CMD_SUCCESS;
 
 	ggsn->cfg.echo_interval = 0;
@@ -681,7 +681,7 @@ DEFUN(cfg_apn_no_gpdu_seq, cfg_apn_no_gpdu_seq_cmd,
 
 DEFUN(cfg_apn_shutdown, cfg_apn_shutdown_cmd,
 	"shutdown",
-	"Put the APN in administrative shut-down\n")
+	"Put the APN in administrative shutdown\n")
 {
 	struct apn_ctx *apn = (struct apn_ctx *) vty->index;
 
@@ -698,7 +698,7 @@ DEFUN(cfg_apn_shutdown, cfg_apn_shutdown_cmd,
 
 DEFUN(cfg_apn_no_shutdown, cfg_apn_no_shutdown_cmd,
 	"no shutdown",
-	NO_STR "Remove the APN from administrative shut-down\n")
+	NO_STR "Remove the APN from administrative shutdown\n")
 {
 	struct apn_ctx *apn = (struct apn_ctx *) vty->index;
 
@@ -796,6 +796,7 @@ static int config_write_ggsn(struct vty *vty)
 			vty_out(vty, " gtp control-ip %s%s", in46a_ntoa(&ggsn->cfg.gtpc_addr), VTY_NEWLINE);
 		if (ggsn->cfg.gtpu_addr.v4.s_addr)
 			vty_out(vty, " gtp user-ip %s%s", in46a_ntoa(&ggsn->cfg.gtpu_addr), VTY_NEWLINE);
+		osmo_tdef_vty_groups_write(vty, " ");
 		llist_for_each_entry(apn, &ggsn->apn_list, list)
 			config_write_apn(vty, apn);
 		if (ggsn->cfg.default_apn)
@@ -1114,6 +1115,8 @@ int ggsn_vty_init(void)
 	install_element(GGSN_NODE, &cfg_ggsn_echo_interval_cmd);
 	install_element(GGSN_NODE, &cfg_ggsn_no_echo_interval_cmd);
 
+	osmo_tdef_vty_groups_init(GGSN_NODE, ggsn_tdef_group);
+
 	install_node(&apn_node, NULL);
 	install_element(APN_NODE, &cfg_description_cmd);
 	install_element(APN_NODE, &cfg_no_description_cmd);
