@@ -441,7 +441,7 @@ int gtp_new(struct gsn_t **gsn, char *statedir, struct in_addr *listen,
 	/* Initialize timers: */
 	(*gsn)->tdef = gtp_T_defs;
 	/* Small hack to properly reset tdef for old clients not using the tdef_group: */
-	osmo_static_assert(gtp_T_defs[0].default_val != 0, first_default_val_not_zero);
+	OSMO_ASSERT(gtp_T_defs[0].default_val != 0);
 	if (gtp_T_defs[0].val == 0)
 		osmo_tdefs_reset((*gsn)->tdef);
 
@@ -593,4 +593,4 @@ int gtp_retranstimeout(struct gsn_t *gsn, struct timeval *timeout)
 	timeout->tv_usec = 0;
 	/* dummy API, deprecated. Return a huge timer to do nothing */
 	return 0;
-}
\ No newline at end of file
+}
